"""
defining a metaclass for tracing function calls and attribute access.
"""

# imports ####################################################################

import types


# basic tracers ##############################################################

def trace(func_name, f, name):
	"""noop wrapping.
	func_name is the name of the function (function like object),
	f         is the actual function
	name      is the name of the class to which the method belongs
	"""
	return f
		
		
class TextualTracer:
	"""Textual function tracer."""
	indent = ""
	
	@classmethod
	def trace(cls, func_name, f, name):
		def traced(*args, **kwargs):
			print "%s%s.%s (args=%s)" % (cls.indent + "> ", name, func_name, args[1:])
			cls.indent += "|"
			result = f(*args, **kwargs) 
			cls.indent = cls.indent[:-1]
			print "%s%s (%s.%s)" % (cls.indent + "< ", result, name, func_name)
			return result
		traced.__name__ = f.__name__
		traced.__doc__ = f.__doc__
		return traced

	
# visual tracer ##############################################################

class VisualTracer:
	"""Visual function tracer."""

	import Tkinter as Tk

	callee = []
		
	def __init__(self):
		self.window = self.Tk.Toplevel()
		self.window.title("visual tracer")
		self.canvas = self.Tk.Canvas(self.window, width=800, height=600)
		self.canvas.pack(fill=self.Tk.BOTH, expand=True)
		
		
	def trace(self, func_name, f, name):
		if func_name is "__init__":
			def traced(*args, **kwargs):
 				print "pre %s.__init__ %s" % (name, args[0])
				result = f(*args, **kwargs)
				print "post %s.__init__" % name
				return result
		else:
			def traced(*args, **kwargs):
				return f(*args, **kwargs)
		traced.func_name = func_name
		traced.__doc__ = f.__doc__	
		return traced
	
	
# tracer metaclass ###########################################################	

trace = TextualTracer.trace
#trace = VisualTracer().trace


class TracerMC(type):
	"""Function call & attribute access tracer metaclass."""
	
	def __new__(cls, name, bases, dict):
		for key, value in dict.iteritems():
			if type(value) is types.FunctionType:
				value = trace(key, value, name)
			elif type(value) is property:
				value = property(trace(key, value.__get__, "<< %s"%name),
				                 trace(key, value.__set__, ">> %s"%name))
			dict[key] = value
				
		return super(TracerMC, cls).__new__(cls, name, bases, dict)


# main #######################################################################

def _test():
	pass

if __name__ == "__main__":
	import sys
	sys.exit(_test())
	
